      SUBROUTINE REFRMM(ELV,RANGEM,DELV,DRANGE)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'LICENSE-BSD3.inc'
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Thu Oct 20, 1988 16:22:02
C     1) REPLACE "COMPILER DOUBLE PRECISION" WITH
C        "IMPLICIT DOUBLE PRECISION (A-H,O-Z)".
C     2) MAKE SURE STATEMENTS DON'T EXTEND BEYOND COLUMN 72.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Mon Oct 31, 1988 13:13:39
C     1) MOVE DATA STATEMENT TO BLKNP.
C     2) REPLACE "REAL" TYPE W/ "D.P." TYPE.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  CORRECTS ELEVATION AND RANGE FOR ATMOSPHERIC REFRACTION.
C  REFERENCE -	J.W.MARINI AND C.W.MURRAY, GODDARD SPACEFLIGHT
C		CENTER REPORT X-591-73-351, NOV. 1973  (CORRECTION
C		OF LASER RANGE TRACKING DATA FOR ATMOSPHERIC
C		REFRACTION AT ELEVATIONS ABOVE 10 DEGREES)
C  INPUT:
C       ELV    - ELEVATION IN RADIANS,
C       RANGEM - 2-WAY RANGE IN METERS.
C     <FROM COMMON ENVPAR>
C       PRESS  - ATMOSPHERIC PRESSURE IN MILLIBARS,
C       TEMP - ATMOSPHERIC TEMPERATURE IN DEGREES KELVIN,
C       HUMID - RELATIVE HUMIDITY IN PERCENT,
C       WMICRN  - LASER WAVELENGTH IN MICRONS.
C     <FROM COMMON SITE>
C       GDLAT  - GEODETIC LATITUDE OF SITE,
C       HGT    - HEIGHT IN METERS OF SITE ABOVE REFERENCE ELLIPSOID.
C
C  OUTPUT:
C       DELV   - ELEVATION CORRECTION IN RADIANS TO BE ADDED TO ELV,
C       DRANGE - RANGE CORRECTION IN METERS TO BE ADDED TO RANGEM (1-WAY).
C
C  REVISION HISTORY:
C       08/23/81 - SEPARATED FROM AZELVR.                               RLR
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

	character*(*) sccsid
	parameter (sccsid = "@(#)refrmm.f	1.2\t06/12/89")

	INCLUDE 'SITE.INC'

	INCLUDE 'ENVPAR.INC'

      DELV = 0.D0
      DRANGE = 0.D0
C  DON'T BOTHER IF BELOW HORIZON OR T OR P ARE PHONY
      IF(ELV.LE.0.D0 .OR. PRESS.EQ.0.D0 .OR. TEMP.EQ.0.D0)GOTO 900
C
C...SATELLITE ELEVATION REFRACTION CORRECTION:
C   (ADAPTED FROM U.T. AEROSPACE UTOPIA SUBROUTINE ELEREF)
C  COMPUTE THE PARTIAL PRESSURE OF WATER VAPOR:
      DELT = TEMP - 273.15D0
      E  = 0.0611D0 * HUMID * 10.D0**(7.5D0*DELT/(237.3D0+DELT)) 
C  FACTOR DEPENDING ON WAVELENGTH (LAMBDA) OF LASER:
	FLAMDA = .9650D0 + 0.0164D0/WMICRN**2 + 0.00028D0/WMICRN**4
C  N SUB S  FACTOR:
      XNS    = 80.343D0 / TEMP * FLAMDA *  (PRESS - E*0.140647D0)
C  Q FACTOR:
      SINELV = DSIN (ELV)
      Q      = 1.D0 / (SINELV + DSQRT(SINELV**2 + 0.0025284D0))
C  DELV IS REFRACTION CORRECTION TO ELEVATION IN RADIANS:
      DELV   = Q * DCOS(ELV) * (6.0116D-4 + 1.9206D-6*(XNS-313.D0))
C
C...LASER RANGE ATMOSPHERIC REFRACTION CORRECTION:
C   (ADAPTED FROM U.T. AEROSPACE UTOPIA SUBROUTINE REFGSGC)
      H1    = 6.11D0 * HUMID * 0.01D0 *
     $      10.D0**(7.5D0*(TEMP-273.15D0)/(237.3D0+TEMP-273.15D0))
      A     = 0.002357D0 * PRESS   +   0.000141D0 * H1
      CLAT2 = DCOS(2.D0*GDLAT)
      RK    = 1.163D0 - 0.00968D0*CLAT2 - 0.00104D0*TEMP +
     1						0.00001435D0*PRESS
      B     = 1.084D-8 * PRESS * TEMP * RK
     $      + 2.D0 * 4.734D-8 * PRESS * PRESS / (TEMP*(3.D0-1.D0/RK))
      FPH   = 1.D0 - 0.0026D0*CLAT2 - 3.1D-7*HGT
      DRANGE= FLAMDA*(A+B) / (FPH*(SINELV+B/((A+B)*(SINELV+0.01D0))))
C
C
900   CONTINUE
C
C	   THE
	   END

